#import <Foundation/Foundation.h>

typedef enum 
{
	kRedColor,
	kGreenColor,
	kBlueColor
} ShapeColor;

typedef struct 
{
	int x, y, width, height;
} ShapeRect;

NSString *colorName (ShapeColor color);
void drawShapes (id shapes[], int count);

NSString *colorName (ShapeColor color)
{
	switch (color) 
    {
		case kRedColor:
			return @"red";
			break;
		case kGreenColor:
			return @"green";
			break;
		case kBlueColor:
			return @"blue";
			break;
	}
	
	return @"no clue";
	
}

@interface Shape : NSObject 
{
    ShapeColor  fillColor;
    ShapeRect   bounds;
}
- (void) setFillColor: (ShapeColor) fillColor;
- (void) setBounds: (ShapeRect) bounds;
- (void) draw;
@end
@implementation Shape
- (void) setFillColor: (ShapeColor) c
{
	fillColor = c;
}

- (void) setBounds: (ShapeRect) b
{
	bounds = b;
}

- (void) draw
{
}

@end // Shape

// --------------------------------------------------
// All about Circles
@interface Circle : Shape
@end // Circle
@implementation Circle
- (void) draw
{
	NSLog (@"drawing a circle at (%d %d %d %d) in %@",
		   bounds.x, bounds.y, 
		   bounds.width, bounds.height,
		   colorName(fillColor));
} // draw
@end // Circle

// --------------------------------------------------
// All about Rectangles
@interface Rectangle : Shape
@end // Rectangle
@implementation Rectangle
- (void) draw
{
	NSLog (@"drawing a rectangle at (%d %d %d %d) in %@",
		   bounds.x, bounds.y, 
		   bounds.width, bounds.height,
		   colorName(fillColor));
} // draw

@end // Rectangle

// --------------------------------------------------
// All about OblateSphereoids
@interface OblateSphereoid : Shape
@end // OblateSphereoid
@implementation OblateSphereoid
- (void) draw
{
	NSLog (@"drawing an egg at (%d %d %d %d) in %@",
		   bounds.x, bounds.y, 
		   bounds.width, bounds.height,
		   colorName(fillColor));
} // draw
@end // OblateSphereoid

// --------------------------------------------------
// All about Triangle
@interface Triangle : Shape 
@end // Triangle
@implementation Triangle
- (void) draw
{
    NSLog (@"drawing a Triangle at (%d %d %d %d) in %@",
		   bounds.x, bounds.y, 
		   bounds.width, bounds.height,
		   colorName(fillColor));
}
@end

// --------------------------------------------------
// Draw the shapes
void drawShapes (id shapes[], int count)
{
	int i;
	
	for (i = 0; i < count; i++) 
    {
		id shape = shapes[i];
		[shape draw];
	}
	
} // drawShapes

// --------------------------------------------------
// The main function.  Make the shapes and draw them
int main (int argc, const char * argv[]) 
{
	id shapes[4];
	
	ShapeRect rect0 = { 0, 0, 10, 30 };
	shapes[0] = [Circle new];
	[shapes[0] setBounds: rect0];
	[shapes[0] setFillColor: kRedColor];
	
	ShapeRect rect1 = { 30, 40, 50, 60 };
	shapes[1] = [Rectangle new];
	[shapes[1] setBounds: rect1];
	[shapes[1] setFillColor: kGreenColor];
	
	ShapeRect rect2 = { 15, 19, 37, 29 };
	shapes[2] = [OblateSphereoid new];
	[shapes[2] setBounds: rect2];
	[shapes[2] setFillColor: kBlueColor];
    
	ShapeRect rect3 = { 47, 32, 80, 50 };
	shapes[3] = [Triangle new];
	[shapes[3] setBounds: rect3];
	[shapes[3] setFillColor: kRedColor];
	
	drawShapes (shapes, 4);
	
	return 0;
	
} // main